# -*- coding: utf-8 -*-

from __future__ import absolute_import

__version__ = '3.5.4'

if __name__ == '__main__':
    import sys
    from bigmler.bigmler import main
    sys.exit(main(sys.argv[1:]))
